@@ -36,7 +36,7 @@ class Application:
         self.toplevel.bind("<Control-o>", self.selectfile)
         self.toplevel.bind("<Control-q>", self.wrapquit)
         self.toplevel.bind("<Control-x>", self.renameFile)
-        self.toplevel.bind("<Control-v>", self.show_connect_VM)
+        self.toplevel.bind("<Control-b>", self.show_connect_VM)
         self.toplevel.bind("<Control-t>", self.takeScreenshot)
         # Initiate the main frame of the application.
         self.frame = tk.Frame(self.toplevel)
@@ -83,7 +83,7 @@ class Application:
         # Define the VM submenu
         self.menu_vm = tk.Menu(self.menu)
         self.menu.add_cascade(menu=self.menu_vm, label='vMachine')
-        self.menu_vm.add_command(label='Connect VM', accelerator='Ctrl-V', command=self.show_connect_VM)
+        self.menu_vm.add_command(label='Connect VM', accelerator='Ctrl-B', command=self.show_connect_VM)
         self.menu_vm.add_command(label='Take screenshot', accelerator='Ctrl-T', command=self.takeScreenshot)
         # Register the menu in the top level widget.
         self.toplevel['menu'] = self.menu
@@ -96,10 +96,32 @@ class Application:
         """Opens an image for editing when passed as a CLI argument upon starting the editor."""
         self.directory = os.path.dirname(path)
         image = os.path.basename(path)
+        # If a json file is opened, we assume that we want to load the needle automatically
+        # and that the needle exists, there we will open and load the needle file.
         if '.json' in image:
             prefix = image.split('.')[0]
             image = prefix + '.png'
-        self.imageName = image    
+            self.imageName = image
+            self.loadNeedle()
+        else:
+            # If however, we start the application with the PNG file, the json needle file
+            # might not exist, therefore we need to perform a test first, before we try to load
+            # it. 
+            # If the test fails, we will at least set the tag from the filename
+            # to save users some time to type. Users can edit it, later and change the file name
+            # from within the application.
+            self.imageName = image
+            jfile = os.path.join(self.directory, self.imageName)
+            # Let's replace the suffix and test if the file exists.
+            pre = jfile.split('.')[0]
+            exists = os.path.exists(f"{pre}.json")
+            # Load the needle if it exists
+            if exists:
+                self.loadNeedle()
+            else:
+                # Or just prefill the tag field from the file name.
+                tag = self.imageName.split('.')[0]
+                self.textField.insert("end", tag)
         self.imageCount = 0
         path = os.path.join(self.directory, self.imageName)
         self.displayImage(path)
@@ -194,29 +216,35 @@ class Application:
         self.typeList = tk.Spinbox(self.jsonFrame, values=["match","ocr","exclude"])
         self.typeList.grid(row=7, column=0, sticky="ew")
         
+        self.matchLabel = tk.Label(self.jsonFrame, text="Match level:")
+        self.matchLabel.grid(row=8, column=0, sticky="w")
+
+        self.matchEntry = tk.Entry(self.jsonFrame, width=30)
+        self.matchEntry.grid(row=9, column=0, sticky="ew")
+
         self.textLabel = tk.Label(self.jsonFrame, text="Tags:")
-        self.textLabel.grid(row=8, column=0, sticky="w")
+        self.textLabel.grid(row=10, column=0, sticky="w")
         
-        self.textField = tk.Text(self.jsonFrame, width=30, height=5)
-        self.textField.grid(row=9, column=0, sticky="ew")
+        self.textField = tk.Text(self.jsonFrame, width=30, height=3)
+        self.textField.grid(row=11, column=0, sticky="ew")
         
         self.jsonLabel = tk.Label(self.jsonFrame, text="Json Data:")
-        self.jsonLabel.grid(row=10, column=0, sticky="w")
+        self.jsonLabel.grid(row=12, column=0, sticky="w")
         
-        self.textJson = tk.Text(self.jsonFrame, width=30, height=10)
-        self.textJson.grid(row=11, column=0, sticky="ew")
+        self.textJson = tk.Text(self.jsonFrame, width=30, height=8)
+        self.textJson.grid(row=13, column=0, sticky="ew")
 
         self.needleLabel = tk.Label(self.jsonFrame, text="Areas in needle: ")
-        self.needleLabel.grid(row=12, column=0, sticky="w")
+        self.needleLabel.grid(row=14, column=0, sticky="w")
 
         self.needleEntry = tk.Entry(self.jsonFrame, width=30)
-        self.needleEntry.grid(row=13, column=0, sticky="w")
+        self.needleEntry.grid(row=15, column=0, sticky="ew")
 
         self.vmLabel = tk.Label(self.jsonFrame, text="VM connection:")
-        self.vmLabel.grid(row=14, column=0, sticky="w")
+        self.vmLabel.grid(row=16, column=0, sticky="w")
         
         self.vmEntry = tk.Entry(self.jsonFrame, width=30)
-        self.vmEntry.grid(row=15, column=0, sticky="w")
+        self.vmEntry.grid(row=17, column=0, sticky="ew")
         self.vmEntry.insert("end","Not connected")
 
 
@@ -351,11 +379,20 @@ class Application:
         try:
             self.needleCoordinates = [self.area[0], self.area[1], self.area[2], self.area[3]]
             typ = self.area[4]
+            # Some needles do not have matches defined, so we must be very careful with this.
+            try:
+                match = self.area[5]
+            except IndexError:
+                match = None
+                
             self.rectangle = self.pictureField.create_rectangle(self.needleCoordinates, outline="red", width=2)
             self.rectangles.append(self.rectangle)
             self.displayCoordinates(self.needleCoordinates)
             self.typeList.delete(0, "end")
             self.typeList.insert("end", typ)
+            if match:
+                self.matchEntry.delete(0, "end")
+                self.matchEntry.insert("end", match)
         except TypeError:
             for r in self.rectangles:
                 self.pictureField.delete(r)
@@ -385,6 +422,7 @@ class Application:
         apos = self.needleCoordinates[2]
         bpos = self.needleCoordinates[3]
         typ = self.typeList.get()
+        match = self.matchEntry.get()
         props = self.propText.get("1.0", "end-1c")
         if "\n" in props:
             props = props.split("\n")
@@ -395,7 +433,11 @@ class Application:
             tags = tags.split("\n")
         if tags == "":
             tags = []
-        coordinates = [xpos, ypos, apos, bpos, typ]
+        # Do not force match onto needle json when not defined
+        if match:
+            coordinates = [xpos, ypos, apos, bpos, typ, match]
+        else:
+            coordinates = [xpos, ypos, apos, bpos, typ]
         self.needle.update(coordinates, tags, props)
         self.textJson.delete("1.0", "end")
         json = self.needle.provideJson()
@@ -521,7 +563,6 @@ class Application:
             self.handler = fileHandler(jsonfile)
             self.handler.readFile()
             jsondata = self.handler.provideData()
-
             self.needle = needleData(jsondata)
             properties = self.needle.provideProperties()
             self.propText.delete("1.0", "end")
@@ -558,7 +599,11 @@ class Application:
         """ Rename the needle PNG file with the top placed tag. """
         tags = self.textField.get("1.0", "end-1c").split("\n")
         future_name = tags[0]
-        if future_name and self.imageName:
+        if not future_name and self.imageName:
+            current = self.returnPath(self.imageName)
+            tag = self.imageName.split('.')[0]
+            self.textField.insert("end", tag)
+        elif future_name and self.imageName:
             future_name = f"{future_name}.png"
             current = self.returnPath(self.imageName)
             new = os.path.join(self.directory, future_name)
@@ -701,7 +746,13 @@ class needleData:
 
     def provideProperties(self):
         """Provide properties."""
-        properties = "\n".join(self.jsonData["properties"])
+        # Some needles do not have properties defined and they do not open in the
+        # application. We will do as if properties were empty (the mostly are anyway)
+        # and the field will be created in the json as we update the needle.
+        try:
+            properties = "\n".join(self.jsonData["properties"])
+        except KeyError:
+            properties = ""
         return properties
     
     def provideTags(self):
@@ -718,9 +769,16 @@ class needleData:
             wide = area["width"]
             high = area["height"]
             typ = area["type"]
+            try:
+                match = area["match"]
+            except KeyError:
+                match = None
             apos = xpos + wide
             bpos = ypos + high
-            areaData = [xpos, ypos, apos, bpos, typ]
+            if match:
+                areaData = [xpos, ypos, apos, bpos, typ, match]
+            else:
+                areaData = [xpos, ypos, apos, bpos, typ]
             self.areaPos += 1
             return areaData
         except IndexError:
@@ -733,9 +791,18 @@ class needleData:
         apos = coordinates[2]
         bpos = coordinates[3]
         typ = coordinates[4]
+        # Again, if no match is defined, the key is missing from the data.
+        try:
+            match = coordinates[5]
+        except IndexError:
+            match = None
         wide = int(apos) - int(xpos)
         high = int(bpos) - int(ypos)
-        area = {"xpos":xpos, "ypos":ypos, "width":wide, "height":high, "type":typ}
+        if match:
+            area = {"xpos":xpos, "ypos":ypos, "width":wide, "height":high, "type":typ, "match":match}
+        else:
+            area = {"xpos":xpos, "ypos":ypos, "width":wide, "height":high, "type":typ}
+
         if type(props) != list:
             props = [props]
         if type(tags) != list:
